﻿////////////////////////////////////////////////////////////////////////
// Модуль Notifier
////////////////////////////////////////////////////////////////////////
var Notifier = (function () {

    // функции обратного вызова
    var registrationStatusChangeCallback = null;
    var fileStatusChangeCallback = null;

    // событие - изменился статус регистрации в ФНС
    var eventRegistrationStatusChange = function (cb) {
        registrationStatusChangeCallback = cb;
    };

    // событие - изменился статус отправки файла в ФНС
    var eventFileStatusChange = function (cb) {
        fileStatusChangeCallback = cb;
    };

    ///////////////////////////////////////////////////////////////////////
    var init = function () {

        var url = $("#NotifierUrl").val();
        var connection = $.hubConnection(url);
        var proxy = connection.createHubProxy('NotifierHub');

        // подключение к хабу
        connection.start({ jsonp: true }).done(function () {
            console.log("Notifier proxy connected to url: " + url);
        });

        // изменился статус Регистрации
        proxy.on('registrationStatusChanged', function (registration_id, status, status_text, status_color, status_list_color) {
            console.log("registrationStatusChanged(): registration id = " + registration_id + "; new status_text = " + status_text);
            if (registrationStatusChangeCallback) {
                registrationStatusChangeCallback(registration_id, status, status_text, status_color, status_list_color);
            }
        });

        // изменился статус Файла
        proxy.on('registrationFileStatusChanged', function (
            registration_id,
            file_id,
            status,
            status_text,
            status_color,
            description,
            request_id,
            document_id) {
            console.log("registrationFileStatusChanged(): registration id = " + registration_id + "; file_id = " + file_id + "; new status = " + status_text);
            if (fileStatusChangeCallback) {
                fileStatusChangeCallback(registration_id, file_id, status, status_text, status_color, description, request_id, document_id);
            }
        });

        // информационное сообщение
        proxy.on('infoNotification', function (message) {
            console.log("infoNotification: message = " + message);
            toastr.success(message);
        });

        // предупреждение
        proxy.on('warningNotification', function (message) {
            console.log("warningNotification: message = " + message);
            toastr.warning(message, "Предупреждение");
        });

        // ошибка
        proxy.on('errorNotification', function (message) {
            console.log("errorNotification: message = " + message);
            toastr.error(message, "Ошибка");
        });

    };
    ///////////////////////////////////////////////////////////////////
    return {
        init: init,
        eventRegistrationStatusChange: eventRegistrationStatusChange,
        eventFileStatusChange: eventFileStatusChange
    };

})();

///////////////////////////////////////////////////////////////////////////////
$(function () {
    Notifier.init();
});