﻿////////////////////////////////////////////////////////////////////////
// Модуль CaseList
////////////////////////////////////////////////////////////////////////
var CaseList = (function () {
    ////////////////////////////////////////////////////////////////////////
    var caseTable = null;
    ////////////////////////////////////////////////////////////////////////
    var Columns = {
        create_date: 0,
        batch_serial: 1,
        serial: 2,
        child_amount: 3,
        status: 4,
        status_color: 5,
        action: 6
    };
    ////////////////////////////////////////////////////////////////////////
    var itemView = function (id) {
        var url = $("#GetItemAjax").val();

        $.get(url, { id: id })
                .done(function (data) {
                    var res = false;
                    if (data !== null && data !== '') {
                        var item = JSON.parse(data);
                        if (item.product_tnved !== null) {
                            $('#divTNVED').html(item.product_tnved);
                        }
                        if (item.serial !== null) {
                            $('#divSn').html(item.serial);
                        }
                        if (item.level !== null) {
                            $('#divLevel').html(item.level);
                        }
                        if (item.company_code !== null) {
                            $('#divCompanyCode').html(item.company_code);
                        }

                        if (item.image !== null && item.image !== '') {
                            var oImg = document.createElement("img");
                            oImg.className = 'code128-img';
                            oImg.setAttribute('src', 'data:image/png;base64,' + item.image);
                            $("#divImage").html(oImg);
                        }
                        $('#caseLabelView').modal('show');
                    }
                })
              .fail(function () {
                  $("#ordersSpinner").hide();
                  $("#ordersError").html('<h3><span class=\"error\"> Ошибка подключения </span></h3>');
                  $("#ordersError").show();
              });
    };
    /////////////////////////////////////////////////////////////////////
    var showHideFilter = function () {
        var show_filter = $('#show_filter').val();
        if (show_filter == 'true') {
            $('#divFilter').show();
        } else {
            $('#divFilter').hide();
        }

        var batch_id = $("#batch_id").val();
        loadCaseTable(batch_id, null, null, null, null, null, null);
    };
    /////////////////////////////////////////////////////////////////////
    var loadCaseTable = function (batch_id, batch_serial, sn, status, filter_date, product_gtin, product_name) {
        // очистить таблицу
        if (caseTable != null) {
            caseTable.api().clear();
        }

        $.fn.dataTable.moment('DD.MM.YYYY HH:mm:ss');
        var url = $("#GetListAjax").val();
        caseTable = $('#caseTable').dataTable({
            language: {
                "url": $("#lang").val()
            },
            "ajax": {
                url,
                    type: "POST",
                data: {
                    batch_id: batch_id,
                    batch_serial: batch_serial,
                    sn: sn,
                    status: status,
                    filter_date: filter_date,
                    product_gtin: product_gtin,
                    product_name: product_name
                }
            },
            "bDestroy": true,
            "bFilter": false,
            "bServerSide": true,
            "bProcessing": true,
            "order": [[0, "desc"]],
            "columnDefs": [
                {
                    render: function (data, type, full, meta) {
                        var child_amount = full[Columns.child_amount];
                        return "<div class='hor-mid'>" + child_amount + "</div>";
                    },
                    targets: Columns.child_amount
                },
                {
                    render: function (data, type, full, meta) {
                        var status_color = full[Columns.status_color];
                        return "<div class='hor-mid list-status-box " + status_color + "'>" + data + "</div>";
                    },
                    targets: Columns.status
                },            
                {
                    "targets": [Columns.status_color],
                    "visible": false,
                    "searchable": false
                },
                {
                    "width": "90",
                    "orderable": false,
                    "targets": -1,
                    "data": null,
                    render: function (data, type, full, meta) {
                        var url = $("#viewDetailsUrl").val() + "/" + data[Columns.action];
                        return "<div class=\"dropdown hor-mid\">" +
                                        "<a href='" + url + "' onClick=\"window.location.href ='" + url + "'\" class=\"dropdown-toggle icon-box infont\" data-placement=\"bottom\" title=\"Детализация\" data-toggle=\"dropdown\">" +
                                            "<i class=\"fa fa-eye table-icons-big\"></i>" +
                                        "</a>" +
                                    "</div>";
                    }
                }
            ]
        });
    };
    /////////////////////////////////////////////////////////////////////
    var onCreateClick = function () {
        $("#btnCreate").click(function () {
            CaseSelectBatch.show();
        });
    };
    /////////////////////////////////////////////////////////////////////
    var batchSelected = function (batch_serial) {
        var url = $("#CreateAjax").val();
        $.post(url,
                {
                    batch_serial: batch_serial
                })
                .done(function (data) {
                    if (data.Result == true) {
                        window.location.href = $("#viewDetailsUrl").val() + "/" + data.Id;
                    } else {
                        toastr.error(data.Description, "Ошибка!");
                    }
                })
                .fail(function () {
                    toastr.error("Ошибка подключения. Обновите страницу.");
                });
    };
    /////////////////////////////////////////////////////////////////////////
    var onSerialNumberReceived = function (serial) {
        var url = $("#GetPackageUrlAjax").val();
        $.post(url, {
            serial: serial
        })
         .done(function (data) {
             if (data != null && data != '' && data.Result == true) {
                 window.location.href = data.url;
             }
         })
         .fail(function () {
             toastr.error("Невозможно выполнить операцию", "Ошибка!");
         });
    };
    ////////////////////////////////////////////////////////////////////////
    var init = function () {

        onCreateClick();

        CaseSelectBatch.init();

        CaseSelectBatch.eventSelectBatch(batchSelected);

        $("#filter_date").datepicker({
            format: 'DD.MM.YYYY',
            language: 'ru'
        });

        showHideFilter();

        $("#btnFilter").click(function () {
            if ($('#show_filter').val() == 'true') {
                $('#show_filter').val('false');
            } else {
                $('#show_filter').val('true');
            }
            showHideFilter();

            $('#batch_serial').val("");
            $('#serial').val("");
            $('#filter_date').val("");
            $('#product_gtin').val("");
            $('#product_name').val("");

        });
        
        $("#btnApplyFilter").click(function () {
            var batch_serial = $('#batch_serial').val();
            var sn = $('#serial').val();
            var status = $('#status_list :selected').val();
            var filter_date = $('#filter_date').val();
            var batch_id = $("#batch_id").val();
            var product_gtin = $("#product_gtin").val();
            var product_name = $("#product_name").val();

            loadCaseTable(batch_id, batch_serial, sn, status, filter_date, product_gtin, product_name);
        });

        Scanner.eventSerialNumber(onSerialNumberReceived);
    };
    ////////////////////////////////////////////////////////////////////////
    return {
        init: init
    };
    ////////////////////////////////////////////////////////////////////////
})();

/////////////////////////////////////////////////////////////////////
$(function () {

    CaseList.init();
    
});
////////////////////////////////////////////////////////////////////////