﻿////////////////////////////////////////////////////////////////////////
// Модуль ShipmentBatch
////////////////////////////////////////////////////////////////////////
var WrapperBatch = (function () {
    // таблица партий
    var table = null;

    var selectedCallback = null;
    var eventSelected = function (cb) {
        selectedCallback = cb;
    };

    // поля таблицы
    var Columns = {
        create_date: 0,
        batch_serial: 1,
        product_gtin: 2,
        product_name: 3,
        sn_amount: 4,
        status: 5,
        status_color: 6,
        production_id: 7,
        action: 8
    };

    // управление показом фильтра
    var showHideFilter = function () {
        var show_filter = $('#show_batch_filter').val();
        if (show_filter == 'true') {
            $('#divBatchFilter').show();
        } else {
            $('#divBatchFilter').hide();
        }
        loadBatchTable(null, null, null, null, null, null, null);
    };

    var selected = function (serial) {
        $('#batchModal').modal('hide');

        if (selectedCallback != null) {
            selectedCallback(serial);
        }
    };

    var show = function () {
        showHideFilter();
    };

    // загрузка таблицы партий
    var loadBatchTable = function (batch_serial, status, filter_date, product_gtin, product_name, sn_amount, operation) {

        $.fn.dataTable.moment('DD.MM.YYYY HH:mm:ss');

        popUpShowProgress("#batchModal");

        $('#batchModal').modal('show');

        var url = $("#GetBatchListAjax").val();

        table = $('#batchTable').dataTable({
            language: {
                "url": $("#lang").val()
            },
            "ajax": {
                url,
                    type: "post",
                    data: {
                        batch_serial: batch_serial,
                        status: status,
                        filter_date: filter_date,
                        product_gtin: product_gtin,
                        product_name: product_name,
                        sn_amount: sn_amount,
                        operation: operation
                    },
                complete: function () {
                    $("#batchModal .modal-dialog").stop(true, true);
                    $("#batchModal .modal-dialog").animate({ "width": "1000px" }, 300, 'linear');
                    popUpShowResults("#batchModal");
                }
            },
            "bDestroy": true,
            "bFilter": false,
            "bServerSide": true,
            "bProcessing": true,
            "order": [[0, "desc"]],
            "columnDefs": [
                {
                    "width": "120",
                    render: function (data, type, full, meta) {
                        return "<div class='text-nowrap'>" + data + "</div>";
                    },
                    targets: Columns.create_date
                },
                {
                    render: function (data, type, full, meta) {
                        var status_color = full[Columns.status_color];
                        return "<div class='hor-mid list-status-box " + status_color + "'>" + data + "</div>";
                    },
                    targets: Columns.status
                },
                {
                    "targets": Columns.production_id,
                    "visible": false,
                    "searchable": false
                },
                {
                    "targets": Columns.status_color,
                    "visible": false,
                    "searchable": false
                },
                {
                    "width": "90",
                    "orderable": false,
                    "targets": -1,
                    "data": null,
                    render: function (data, type, full, meta) {
                        var batch_serial = data[Columns.batch_serial];
                        return "<div class=\"dropdown hor-mid\">" +
                                        "<a onclick=\"WrapperBatch.selected('" + data[Columns.batch_serial] + "')\" class=\"dropdown-toggle icon-box infont\" data-placement=\"bottom\" title=\"Выбрать\" data-toggle=\"dropdown\">" +
                                            "<i class=\"fa fa-hand-pointer-o table-icons-big\"></i>" +
                                        "</a>" +
                                    "</div>";
                    }
                }
            ]
        });
    };

    // инициализация модуля
    var init = function () {
        // установить формат даты для фильтра
        $("#batch_filter_date").datepicker({
            format: 'dd.mm.yyyy',
            language: 'ru'
        });

        $('#batchModal').on('hidden.bs.modal', function () {
            popUpHideAll("#batchModal");
        });

        $('#batchModal').on('hidden.bs.modal', function () {
            $("#batchModal .modal-dialog").stop(true, true);
            $("#batchModal .modal-dialog").animate({ "width": "500px" }, 100, 'linear');
        });

        $("#btnBatchApplyFilter").click(function () {
            var batch_serial = $('#batch_serial').val();
            var status = $('#status_list :selected').val();
            var filter_date = $('#batch_filter_date').val();
            var product_gtin = $('#product_gtin').val();
            var product_name = $('#product_name').val();
            var sn_amount = $('#sn_amount').val();
            var operation = $('#operation_list :selected').val();

            loadBatchTable(batch_serial, status, filter_date, product_gtin, product_name, sn_amount, operation);
        });

        $("#btnBatchFilter").click(function () {
            if ($('#show_batch_filter').val() == 'true') {
                $('#show_batch_filter').val('false');
            } else {
                $('#show_batch_filter').val('true');
            }
            showHideFilter();

            // очистить фильтр
            $('#batch_serial').val("");
            $('#filter_date').val("");
            $('#sn_amount').val("");

        });
    };

    // экспорт функций, доступных за пределами модуля
    return {
        init: init,
        show: show,
        selected: selected,
        eventSelected: eventSelected
    };
})();
///////////////////////////////////////////////////////////////////////////////