﻿////////////////////////////////////////////////////////////////////////
// Модуль WrapperDetails
////////////////////////////////////////////////////////////////////////
var WrapperDetails = (function () {
    /////////////////////////////////////////////////////////////////////
    var childTable = null; // таблица вложений
    var linkedTables = null; // связанные таблицы
    var addedItems = new Set(); // список уникальных идентификаторов добавленных вложений
    var removedItems = new Set(); // список уникальных идентификаторов удаленных вложений
    /////////////////////////////////////////////////////////////////////
    // список полей таблицы вложений
    var ChildTableColumns = {
        id: 0,
        type_text: 1,
        serial: 2,
        status_text: 3,
        status_color: 4,
        action: 5
    };
    /////////////////////////////////////////////////////////////////////
    var childView = function (id) {
        var url = $("#ChildView").val();
        window.location.href = url + "/" + id;
    };
    /////////////////////////////////////////////////////////////////////
    // Удалить вложение по идентификатору
    var removeChildById = function (child_id) {
        var rowId = childTable.fnFindCellRowIndexes(child_id, ChildTableColumns.id);
        removeChild(rowId);
    };
    /////////////////////////////////////////////////////////////////////
    // Удалить вложение по серийному номеру
    var removeChildBySerial = function (serial) {
        var rowId = childTable.fnFindCellRowIndexes(serial, ChildTableColumns.serial);
        removeChild(rowId);
    };
    /////////////////////////////////////////////////////////////////////
    var removeChild = function (rowId) {
        if (rowId) {
            rowId.forEach(function (item, i, arr) {
                var serial = childTable.api().cell(item, ChildTableColumns.serial).data();
                var id = childTable.api().cell(item, ChildTableColumns.id).data();
                childTable.api().row(item).remove().draw();
                toastr.info("Упаковка № " + serial + " удалена");

                // актуализировать списки добавленных / удаленных вложений
                removedItems.add(parseInt(id));
                addedItems.delete(parseInt(id));
            });
        }
        // актуализировать количество вложений
        $("#child_amount").val(getChildCount());
        $("#childTableChanged").val(true);

        checkStatus();
    };
    /////////////////////////////////////////////////////////////////////
    var getChildCount = function () {
        return childTable.fnSettings().fnRecordsTotal();
    };
    /////////////////////////////////////////////////////////////////////
    var childRetrieve = function (id, message) {
        // удалить вложение по идентификатору
        removeChildById(id);
    };
    /////////////////////////////////////////////////////////////////////
    var menuView = function (url) {
        if (url) {
            window.location.href = url;
        }
    };
    ///////////////////////////////////////////////////////////////////
    var addViewOnly = function (id) {
        var url = getChildUrl(id);
        return "<div class=\"dropdown table_align_center\">" +
                    "<a href=\"" + url + "\" onclick=WrapperDetails.menuView('" + url + "') class=\"dropdown-toggle icon-box infont\" data-placement=\"bottom\" title=\"Детализация\" data-toggle=\"dropdown\">" +
                        "<i class=\"fa fa-eye table-icons-big\"></i>" +
                    "</a>" +
                "</div>";
    }
    /////////////////////////////////////////////////////////////////////
    var addDropDownMenu = function (menu) {
        var line = "<div class=\"dropdown hor-mid \">" +
                        "<a class=\"dropdown-toggle icon-box infont\" data-placement=\"bottom\" title=\"Действие\" href=\"#\" data-toggle=\"dropdown\">" +
                            "<i class=\"fa fa-hand-pointer-o table-icons-big\"></i>" +
                        "</a>" +
                        "<ul class=\"dropdown-menu hdropdown notification animated flipInX extra-margin-left-xl\">" +

                                menu +

                        "</ul>" +
                    "</div>";
        return line;
    };
    /////////////////////////////////////////////////////////////////////
    var getViewMenuItem = function (id) {
        return "<li>" +
                    "<a onclick=\"WrapperDetails.childView(" + id + ")\" class=\"m-r-sm\" data-toggle=\"tooltip\" data-placement=\"bottom\" title=\"\">" +
                        "<i class=\"fa fa-eye table_icons action-menu-item\"></i>Детализация" +
                    "</a>" +
                "</li>";
    };
    /////////////////////////////////////////////////////////////////////
    var getRetrieveViewMenuItem = function (id, message) {
        return "<li>" +
                    "<a onclick=\"WrapperDetails.childRetrieve('" + id + "','" + message + "')\" class=\"m-r-sm\" data-toggle=\"tooltip\" data-placement=\"bottom\" title=\"\">" +
                        "<i class=\"fa fa-upload table_icons action-menu-item\"></i>Извлечь" +
                    "</a>" +
                "</li>";
    };
    /////////////////////////////////////////////////////////////////////
    var checkStatus = function () {
        hideItem("#btnDelete");
        var status = $("#status").val();
        if (status == PackageStatus.Aggregated) {
            showItem("#btnUnpack");
            showItem("#btnRetrieve");
            showItem("#divParent");            
        } else if (status == PackageStatus.Printed) {
            showItem("#btnUnpack");
            hideItem("#btnRetrieve");
            hideItem("#divParent");
        } else if (status == PackageStatus.Shipped) {
            hideItem("#btnRetrieve");
            hideItem("#btnUnpack");
            hideItem("#divParent");
            hideItem("#btnAddChild");
            hideItem("#btnSave");
        } else if (status == PackageStatus.Empty) {
            showItem("#btnDelete");
            hideItem("#btnRetrieve");
            hideItem("#btnUnpack");
        } else {
            hideItem("#btnRetrieve");
            hideItem("#btnUnpack");
            hideItem("#divParent");
        }

        if (status != PackageStatus.Shipped) {
            if (parseInt($("#child_amount").val()) < parseInt($("#size").val())) {
                showItem("#btnAddChild");
            } else {
                hideItem("#btnAddChild");
            }

            // скрыть / показать кнопку сохранения
            if ($("#childTableChanged").val() == 'true') {
                showItem("#btnSave");
            } else {
                hideItem("#btnSave");
            }
        }        
    };
    /////////////////////////////////////////////////////////////////////
    var getMenu = function (id, message) {
        var status = $("#status").val();
        var menu = "";
        if (status == PackageStatus.Aggregated ||
            status == PackageStatus.Printed ||
            status == PackageStatus.Empty) {
            menu =
                addDropDownMenu(
                    getViewMenuItem(id) +
                    getRetrieveViewMenuItem(id, message));
        } else {
            menu = addViewOnly(id);
        }
        return menu;
    };
    /////////////////////////////////////////////////////////////////////
    var renderMenu = function (id, message) {
        return getMenu(id, status, message);
    };
    /////////////////////////////////////////////////////////////////////
    var loadChildTable = function () {

        if (childTable == null) {
            childTable = $('#childTable').dataTable({
                language: {
                    "url": $("#lang").val()
                },
                "order": [[0, "asc"]],
                "bPaginate": false,
                "bFilter": false,
                "info": false,
                "bProcessing": false,
                "autoWidth": false,
                "columnDefs": [
                    {
                        render: function (data, type, full, meta) {
                            return "<div class='hor-mid list-status-box " + full[ChildTableColumns.status_color] + "'>" + data + "</div>";
                        },
                        targets: [ChildTableColumns.status_text]
                    },
                    {
                        "targets": [ChildTableColumns.status_color],
                        "visible": false,
                        "searchable": false
                    },
                    {
                        "targets": [ChildTableColumns.id],
                        "visible": false,
                        "searchable": false
                    },
                    {
                        "orderable": false,
                        "targets": -1,
                        "data": null,
                        render: function (data, type, full, meta) {
                            var message = data[ChildTableColumns.serial] + " (" + data[ChildTableColumns.type_text] + ")";
                            var id = data[ChildTableColumns.id];
                            return renderMenu(id,message);
                        }
                    }
                ]
            });
        }

        var url = $("#GetChildListAjax").val();
        var id = $("#id").val();

        $.get(url, { id: id })
           .done(function (data) {
               if (data.Result == true) {
                   childTable.api().clear().draw();
                   childTable.api().rows.add(data.ChildList).draw();
               }
           })
         .fail(function () {
             // todo
         });
    };
    /////////////////////////////////////////////////////////////////////
    var updateData = function (item) {

        $("#status").val(item.status);
        $("#status_text").text(item.status_text);
        $("#status_text").removeClass();
        $("#status_text").addClass("control-label status-box ");
        $("#status_text").addClass(item.status_color);

        $("#child_amount").val(item.child_amount);
        

        checkStatus();
    };
    ///////////////////////////////////////////////////////////////////////
    // проверить, есть ли строка с таким серийным номеров в таблице
    var checkIfExists = function (serial) {
        var rowId = childTable.fnFindCellRowIndexes(serial, ChildTableColumns.serial);
        var res = rowId != null && rowId.length > 0;
        return res;
    };
    ///////////////////////////////////////////////////////////////////////
    // Достать строки с серийным номером
    var getRowsBySerial = function (serial) {
        return childTable.fnFindCellRowIndexes(serial, ChildTableColumns.serial);
    };
    /////////////////////////////////////////////////////////////////////////
    // Добавить новое вложение в таблицу
    var addRecordToChildTable = function (item) {
        childTable.api().row.add([
                item.id,
                item.type_text,
                item.serial,
                item.status_text,
                item.status_color,
                item.id
        ]).draw();

        // актуализировать списки добавленных / удаленных вложений
        addedItems.add(parseInt(item.id));
        removedItems.delete(parseInt(item.id));

        // актуализировать количество вложений
        $("#child_amount").val(getChildCount());
        $("#childTableChanged").val(true);

        checkStatus();

        $('html, body').animate({
            scrollTop: $("#divChildren").offset().top
        }, 500);

        toastr.info("Упаковка № " + item.serial + " добавлена");
    }
    /////////////////////////////////////////////////////////////////////
    var addChild = function (serial) {
        var status = $("#status").val();
        if (status != PackageStatus.Shipped) {
            if (serial != "") {
                // проверить, есть ли свободное место
                if (getChildCount() < parseInt($("#size").val())) {
                    var url = $("#AddChildAjax").val();
                    var id = $("#id").val();
                    $.post(url, {
                        id: id,
                        serial: serial
                    })
                    .done(function (data) {
                        if (data.Result == true) {
                            if (data.Child != null) {
                                if (!checkIfExists(data.Child.serial)) {
                                    // добавить новую запись в таблицу                                     
                                    addRecordToChildTable(data.Child);
                                } else {
                                    toastr.error("Упаковка № " + data.Child.serial + " уже есть в списке", "Ошибка!");
                                }
                            }
                        }
                        else {
                            toastr.error(data.Description, "Ошибка!");
                        }
                    })
                    .fail(function () {
                        toastr.error("Невозможно выполнить операцию", "Ошибка!");
                    });

                } else {
                    toastr.error("Нет свободного места", "Ошибка!");
                }
            } else {
                toastr.error("Серийный номер не задан", "Ошибка!");
            }
        }
    };
    /////////////////////////////////////////////////////////////////////
    // сохранить добавленные / удаленные вложения
    var onSaveClick = function () {
        $('#btnSave').click(function () {
            var url = $("#SaveAjax").val();
            var status = $("#status").val();
            var id = $("#id").val();

            $.post(url, {
                id: id,
                status: status,
                addedItems: Array.from(addedItems),
                removedItems: Array.from(removedItems)
            })
            .done(function (data) {
                if (data != null && data != '') {
                    if (data.Result == true) {
                        // обновить статус бандероли
                        $("#childTableChanged").val(false);
                        addedItems.clear();
                        removedItems.clear();

                        updateData(data.Item);

                        loadChildTable();

                        // загрузить таблицу связанных объектов
                        LinkedTables.load(id, $("#LoadLinkedTablesAjax").val());

                        toastr.info(data.Description);

                    } else {
                        toastr.error(data.Description, "Ошибка");
                    }
                }
            })
            .fail(function () {
                toastr.error('Ошибка выполнения задания', "Ошибка");
            });
            
        });
    };
    /////////////////////////////////////////////////////////////////////
    var getChildUrl = function (id) {
        return $("#ChildView").val() + "/" + id;
    }
    /////////////////////////////////////////////////////////////////////
    var onAddChildClick = function () {
        $('#btnAddChild').click(function () {
            $('#addChildModal').modal('show');
        });
    };

    /////////////////////////////////////////////////////////////////////
    var onDeleteClick = function () {
        $('#btnDelete').click(function () {
            CommonConfirm.show('Удалить бандероль № ' + $("#serial").val() + '?', onDeleteConfirmed);
        });
    };
    /////////////////////////////////////////////////////////////////////
    var onDeleteConfirmed = function () {
        var url = $("#DeleteAjax").val();
        var id = $("#id").val();

        $.post(url, {
            id: id
        })
      .done(function (data) {
          if (data != null && data != '') {
              if (data.Result == true) {
                  window.location.href = $("#Index").val();
              }
              else {
                  toastr.error(data.Description, "Ошибка!");
              }
          }
          else {
              toastr.error("Невозможно выполнить операцию", "Ошибка!");
          }
      })
      .fail(function () {
          toastr.error("Невозможно выполнить операцию", "Ошибка!");
      });
    };
    /////////////////////////////////////////////////////////////////////    
    var onSerialNumberReceived = function (serial) {        
        if (checkIfExists(serial)) {
            toastr.error("Вложение № " + serial + " уже существует");
        } else {
            // добавить вложение
            addChild(serial);
        }
    };
    /////////////////////////////////////////////////////////////////////////
    var onUnpackClick = function () {
        $('#btnUnpack').click(function () {
            CommonConfirm.show('Распаковать бандероль № ' + $("#serial").val() + '?', onUnpackConfirm);
        });
    };
    /////////////////////////////////////////////////////////////////////
    var onRetrieveClick = function () {
        $('#btnRetrieve').click(function () {
            CommonConfirm.show('Извлечь бандероль № ' + $("#serial").val() + '?', onRetrieveConfirmed);
        });
    };
    /////////////////////////////////////////////////////////////////////
    var onRetrieveConfirmed = function () {

        var url = $("#RetrieveAjax").val();
        var id = $("#id").val();

        $.post(url, {
            id: id
        })
      .done(function (data) {
          if (data != null && data != '') {

              if (data.Result == true) {

                  // обновить статус
                  if (data.Item != null) {

                      $("#status").val(data.Item.status);
                      $("#status_text").text(data.Item.status_text);
                      $("#status_text").removeClass();
                      $("#status_text").addClass("control-label status-box ");
                      $("#status_text").addClass(data.Item.status_color);                      

                      checkStatus();

                      toastr.info("Бандероль извлечена");
                  }
              }
              else {
                  toastr.error(data.Description, "Ошибка!");
              }
          }
          else {
              toastr.error("Невозможно выполнить операцию", "Ошибка!");
          }
      })
      .fail(function () {
          toastr.error("Невозможно выполнить операцию", "Ошибка!");
      });
    };
    /////////////////////////////////////////////////////////////////////
    var collapsExpand = function () {
        $("#btnCollapseExpand").click(function () {
            var collapsed = $("#collapseAll").val();
            if (collapsed == 1) {
                $("#btnCollapseExpand").html("Свернуть");
                $("#collapseAll").val(0);
            } else {
                $("#btnCollapseExpand").html("Развернуть");
                $("#collapseAll").val(1);
            }

            $("#wrapperForm .showhide").each(function () {
                $(this).click();
            });
        });
    };
    /////////////////////////////////////////////////////////////////////
    var onUnpackConfirm = function () {

        var url = $("#UnpackAjax").val();
        var id = $("#id").val();

        $.post(url, {
            id: id
        })
      .done(function (data) {
          if (data != null && data != '') {

              if (data.Result == true) {

                  // обновить статус
                  if (data.Item != null) {

                      updateData(data.Item);

                      loadChildTable();

                      // загрузить таблицу связанных объектов
                      LinkedTables.load(id, $("#LoadLinkedTablesAjax").val());

                      toastr.info("Бандероль распакована");
                  }
              }
              else {
                  toastr.error(data.Description, "Ошибка!");
              }
          }
          else {
              toastr.error("Невозможно выполнить операцию", "Ошибка!");
          }
      })
      .fail(function () {
          toastr.error("Невозможно выполнить операцию", "Ошибка!");
      });
    };   
    /////////////////////////////////////////////////////////////////////
    var init = function () {

        $("#childTableChanged").val(false);

        WrapperAddChild.eventAddChild(addChild);

        WrapperSns.eventSelected(addChild);

        Scanner.eventSerialNumber(onSerialNumberReceived);

        // загрузить таблицу связанных объектов
        LinkedTables.load($("#id").val(), $("#LoadLinkedTablesAjax").val());

        onSaveClick();

        onDeleteClick();

        collapsExpand();

        onAddChildClick();

        onUnpackClick();

        onRetrieveClick();

        // загрузка вложений
        loadChildTable();

        // актуализация полей в соответствии со статусом
        checkStatus();

        $('.panel-heading').on('click', function (event) {
            event.stopPropagation();
            $(this).find(".showhide").click();
        });
    };
    /////////////////////////////////////////////////////////////////////////////
    return {
        init: init,
        childView: childView,
        menuView: menuView,
        childRetrieve: childRetrieve
    };
})();

/////////////////////////////////////////////////////////////////////////////////////////////////
$(function () {

    WrapperDetails.init();
    WrapperAddChild.init();
    WrapperSns.init();
});